/*------------------------------------------------------------------------------------------------------------------------------------------------------>
<!-------------------- CUBASIS USER GUIDE SEARCH ------------------------------------------------------------------------------------------------------->
<!-------------------- v.3.5_1.7_v4  - last modified: 23.10.2013 --------------------------------------------------------------------------------------->
<!-------------------- © 2013 STEINBERG MEDIA TECHNOLOGY GmbH ----------------------------------------------------------------------------------------->
<!------------------------------------------------------------------------------------------------------------------------------------------------------*/

function search(event) {
	if (isFunctionKey(event)) {
		return;
	}


	var results = getSearchResults(document.getElementsByName("input")[0].value);
	document.getElementById("searchRes").innerHTML = "";
	document.getElementById("searchRes").appendChild(results);
}

function checkEnter(event) {
	if (event.keyCode == 13) {
		document.getElementsByName("input")[0].blur();
		event.preventDefault();
		return false;
	}

}

function getVersionString() {
	var version = QueryString.version;
	if (typeof version != 'undefined') {
		return "?version=" + version;
	}
	return "";
}

function getSearchResults(term) {
	var increment = 1;
	var match = false;
	var tag = true;
	var op = true;
	var display;
	var inputLength = term.length;
	var threshold = inputLength / 2;
	var resultArray = new Array();
	var suggestArray = new Array();

	for (i = 0; i <= inputLength; i++) {
		resultArray[i] = new Array();	
	}

	var resultBox = document.createElement("section");
	resultBox.id = "results";

	loop();
	function loop() {
		for (var snippet in snippets) { //Iterate through all snippets
			for(var keyword in snippets[snippet]["keywords"]) { //Iterate through all keywords of a given snippet
				var distance = mrlevenshtein(term, getKeywordPart(snippets[snippet]["keywords"][keyword]));
				if (distance < threshold) { //The keyboard matched to this snippet close enough -> Add this to result array
					addMatchToResultArray(snippet, keyword, distance);
					match = true;
					break;
				}
			}
		}
	}

	if (match) {
		displayResults();
	} else {
		displayNothingFound();
	}

	function displayNothingFound() {
		var s_div = document.createElement("div");
		s_div.innerHTML = nothingFound();
		resultBox.appendChild(s_div);
	}

	//resultArray[distance to search term][weighting factor]["snip"]
	function addMatchToResultArray(snippet, keywordIndex, distance) {
		var keyword = snippets[snippet]["keywords"][keywordIndex]; //Get the matched keyword
		var index = getWeightingFactor(keyword);

		if (resultArray[distance][index] == null) {
			writeSnippetAndKeyword(distance, index, snippet, keywordIndex);
		} else { //Append this result at the end of the results with the given weighting factors
			while (resultArray[distance][index + increment] != null) {
				increment++;
			}
			writeSnippetAndKeyword(distance, index + increment, snippet, keywordIndex);
		}
	}

	function getKeywordPart(string) {
		return string.substr(0, string.indexOf("%"));
	}

	function getWeightingFactor(string) {
		return Number(string.substr(string.indexOf("%") + 1));
	}

	function writeSnippetAndKeyword(distance, index, snippet, keywordIndex) {
		resultArray[distance][index] = new Array();
		resultArray[distance][index]["snip"] = snippet;
		resultArray[distance][index]["key"] = keywordIndex;
	}

	function createDivForSnippet(snippet) {
		var result = document.createElement("div");
			result.setAttribute('class','snippet');
			result.innerHTML = getSnippet(
				snippet["heading"],
				snippet["image"],
				snippet["text"],
				//snippet["id"],
				snippet["link"]);
		return result;
	}

	function displayResults() {
		var search_results = document.createElement("div");
		appendMatchesWithDistanceZero(search_results);
		insertCloseMatches(search_results);
		resultBox.appendChild(search_results);
	}

	function appendMatchesWithDistanceZero(search_results) {
		for(var i = 0; i < resultArray[0].length; i++) {
			if (resultArray[0][i] != null)
				search_results.appendChild(createDivForSnippet(snippets[resultArray[0][i]["snip"]]));
		}
	}

	function insertCloseMatches(search_results) {
		// Insert resultsnippets for close matching (Levenshtein-Distance >= 1)
		for (var distance = 1; distance <= inputLength; distance++) {
			for(var m = 0; m < resultArray[distance].length; m++) {
				var currentResult = resultArray[distance][m];

				if (currentResult != null) {
					var currentSnippet = currentResult["snip"];

					//insert   "typo? - do you mean...? "   - only once!
					var tempKeyword = getKeywordPart(snippets[currentSnippet]["keywords"][currentResult["key"]]);
					if (suggestArray.indexOf(tempKeyword) == -1) { //Already suggested that keyword
						search_results.appendChild(suggest(snippets[currentSnippet]["keywords"][currentResult["key"]]));
					}
					suggestArray.push(tempKeyword);
					search_results.appendChild(createDivForSnippet(snippets[currentSnippet]));
				}
			}
		}
	}

	/*----------------------------------------------------------------------
	** returns innerHTML for ResultHeader
	**----------------------------------------------------------------------*/
	/*function resultHeader() {
		var h2 = document.createElement("h2");
		h2.id = "resultheader";

		var text = document.createTextNode("Search Results");
		h2.appendChild(text);
		return h2;
	}*/

	/*----------------------------------------------------------------------
	** returns innerHTML when nothing found
	**----------------------------------------------------------------------*/
	function nothingFound() {
		return '<h2 id="searchfail">Nothing found</h2>';
	}

	/*----------------------------------------------------------------------
	** returns innerHTML for each snippet
	**----------------------------------------------------------------------*/
	function getSnippet(heading, image, text, link) {
		return '<h2><a href="' + link + getVersionString() + '"> &raquo;  ' + heading + '</a></h2>' +
			'<div class="resultsnippetcontainer">'+
				'<div>' + text + '</div>' +
				'<img src=" ' + image + ' " />' +
			'</div>';
	}

	/*----------------------------------------------------------------------
	** returns suggestion
	**----------------------------------------------------------------------*/
	function suggest(sugg) {
		s = sugg.substring(0, sugg.indexOf("%"));

		var h2 = document.createElement("h2");
		var text = document.createTextNode('Do you mean ');
		h2.appendChild(text);

		var em = document.createElement("em");
		var emText = document.createTextNode(s);
		em.appendChild (emText);
		h2.appendChild(em);

		h2.appendChild(document.createTextNode("?"));
		return h2;
	}

	return resultBox;
}